<?php
/*
Plugin Name: WordPress HTTPS
Plugin URI: http://mvied.com/projects/wordpress-https/
Description: Force HTTPS on only certain pages. Fix partially encrypted errors. Disable WordPress from changing links to HTTPS. And more!
Author: Mike Ems
Version: 1.6.5
Author URI: http://mvied.com/
*/

/**
* Class for the WordPress plugin WordPress HTTPS
*
* @author  Mike Ems
* @package WordPressHTTPS
* @copyright Copyright 2010
*/

if ( !class_exists('WordPressHTTPS') ) {
 class WordPressHTTPS {

  /**
   * Plugin version
   *
   * @var int
   */
  var $plugin_version = '1.6.5';

  /**
   * Plugin URL
   *
   * @var string
   */
  var $plugin_url;

  /**
   * HTTP URL
   *
   * @var string
   */
  var $http_url;

  /**
   * HTTPS URL
   *
   * @var string
   */
  var $https_url;

  /**
   * Default options
   *
   * @private array
   */
  private $options_default;

  /**
   * Initialize plugin
   *
   * @param none
   * @return void
   */
  function __construct() {
   // Assign plugin_url
   if ( version_compare( get_bloginfo('version'), '2.8', '>=' ) ) {
    $this->plugin_url = plugins_url('', __FILE__);
   } else {
    $this->plugin_url = WP_PLUGIN_URL . '/' . plugin_basename(dirname(__FILE__));
   }

   // Assign HTTP URL
   $this->http_url = get_option('home');

   // Assign HTTPS URL and prevent WordPress from causing a redirect loop if using Shared SSL
   if (get_option('wordpress-https_sharedssl') == 1 && get_option('wordpress-https_sharedssl_host') != '') {
    $this->https_url = get_option('wordpress-https_sharedssl_host');

    // Prevent WordPress from causing a redirect loop if using Shared SSL
    remove_filter('template_redirect', 'redirect_canonical');
   } else {
    $this->https_url = $this->replace_http($this->http_url);
   }

   // Define default options
   $this->options_default = array(
    'wordpress-https_internalurls'         => 1, // Force internal URL's to HTTPS
    'wordpress-https_externalurls'         => 0, // Force external URL's to HTTPS
    'wordpress-https_bypass'               => 0, // Bypass option to check if external elements can be loaded via HTTPS
    'wordpress-https_disable_autohttps'    => 0, // Disable the feature in WordPress 3.0+ that makes all links HTTPS if the page is being viewed via HTTPS.
    'wordpress-https_exclusive_https'      => 0, // Exclusively force SSL on posts and pages with the `Force SSL` option checked.
    'wordpress-https_frontpage'            => 0, // Force SSL on front page
    'wordpress-https_sharedssl'            => 0, // Enable Shared SSL
    'wordpress-https_sharedssl_host'       => '' // Hostname for Shared SSL
   );

   if ( is_admin() ) {
    // Add admin menus
    add_action('admin_menu', array(&$this, 'menu'));

    // Load on plugins page
    if ( 'plugins.php' == $GLOBALS['pagenow'] ) {
     add_filter( 'plugin_row_meta', array( &$this, 'plugin_links' ), 10, 2);
    }

    // Load on Settings page
    if (@$_GET['page'] == 'wordpress-https') {
     wp_enqueue_script('jquery-form', $this->plugin_url . '/js/jquery.form.js', array('jquery'), '2.47', true);
     wp_enqueue_script('wordpress-https', $this->plugin_url . '/js/admin.js', array('jquery'), $this->plugin_version, true);
     wp_enqueue_style('wordpress-https', $this->plugin_url . '/css/admin.css', $this->plugin_version, true);

     // Set default options
     foreach ( $this->options_default as $option => $value ) {
      if ( get_option($option) === false ) {
       add_option($option, $value);
      }
     }
    }
   } else {
    // If not admin page, check to see if the page needs to be redirected
    add_action('template_redirect', array(&$this, 'check_https'));
   }

   // Filter HTTPS from links in WP 3.0+
   if ( get_option('wordpress-https_disable_autohttps') == 1 ) {
    add_filter('page_link', array(&$this, 'replace_https'));
    add_filter('post_link', array(&$this, 'replace_https'));
    add_filter('category_link', array(&$this, 'replace_https'));
    add_filter('get_archives_link', array(&$this, 'replace_https'));
    add_filter('tag_link', array(&$this, 'replace_https'));
    add_filter('search_link', array(&$this, 'replace_https'));
    add_filter('home_url', array(&$this, 'replace_https'));
    add_filter('bloginfo', array(&$this, 'bloginfo'), 10, 2);
    add_filter('bloginfo_url', array(&$this, 'bloginfo'), 10, 2);
   }

   // Add 'Force SSL' checkbox to add/edit post pages
   add_action('post_submitbox_misc_actions', array(&$this, 'post_checkbox'));
   add_action('save_post', array(&$this, 'post_save'));

   // Start output buffering
   add_action('plugins_loaded', array(&$this, 'buffer_start'));

   // End output buffering
   //add_action('shutdown', array(&$this, 'buffer_end'));
  }

  /**
   * Process output buffer
   *
   * @param string
   * @return string
   */
  function process($buffer) {
   if ( $this->is_ssl() ) {
    preg_match_all('/\<(script|link|img|input)[^>]+((http|https):\/\/[\/-\w\.]+)[^>]+>/im',$buffer,$matches);

    for ($i = 0; $i<=sizeof($matches[0]); $i++) {
     $html   = $matches[0][$i];
     $type   = $matches[1][$i];
     $url    = $matches[2][$i];
     $scheme = $matches[3][$i];

     if ( ( $type == 'link' && strpos($html, 'stylesheet') !== false ) || ( $type == 'input' && strpos($html, 'image') !== false ) || $type == 'script' || $type == 'img' ) {
      if ( strpos($html,$this->http_url) !== false && get_option('wordpress-https_internalurls') == 1 ) {
       $buffer = str_replace($html, str_replace($this->http_url, $this->https_url, $html), $buffer);
      } else if ( get_option('wordpress-https_externalurls') == 1 ) {
       if ( get_option('wordpress-https_bypass') == 1 ) {
        $buffer = str_replace($html, str_replace($this->http_url, $this->https_url, $html), $buffer);
       } else if (@file_get_contents($this->replace_http($url))) {
        $buffer = str_replace($html, str_replace($this->http_url, $this->https_url, $html), $buffer);
       }
      }
     }
    }
   }
   return $buffer;
  }

  function is_ssl() {
   if ( get_option('wordpress-https_sharedssl') == 1 ) {
    if ( strpos($this->https_url, $_SERVER['HTTP_X_FORWARDED_SERVER']) !== false ) {
     return true;
    } else {
     return false;
    }
   } else {
    return is_ssl();
   }
  }

  /**
   * Checks to see if the current page needs to be redirected
   *
   * @param none
   * @return void
   */
  function check_https() {
   global $post;
   if (is_front_page() && get_option('show_on_front') == 'posts') {
    if (get_option('wordpress-https_frontpage') == 1 && !$this->is_ssl() ) {
     $this->redirect(true);
    } else if (get_option('wordpress-https_frontpage') != 1 && $this->is_ssl()) {
     $this->redirect(false);
    }
   } else if ( is_single() || is_page() || is_front_page() || is_home() ) {
    if ( !$this->is_ssl() ) {
     $forceSSL = get_post_meta($post->ID, 'force_ssl');
     if ($forceSSL) {
      $this->redirect(true);
     }
    } else if ( get_option('wordpress-https_exclusive_https') == 1 ) {
     $forceSSL = get_post_meta($post->ID, 'force_ssl');
     if (!$forceSSL) {
      $this->redirect(false);
     }
    }
   }
  }

  /**
   * Redirects page to HTTP or HTTPS accordingly
   *
   * @param boolean $ssl
   * @return void
   */
  function redirect($ssl = true) {
   if ( !$this->is_ssl() && $ssl == true ) {
    wp_redirect($this->https_url . $_SERVER['REQUEST_URI']);
    exit();
   } else if ($this->is_ssl() && $ssl == false) {
    wp_redirect($this->http_url . $_SERVER['REQUEST_URI']);
    exit();
   }
  }

  /**
   * Add 'Force SSL' checkbox to add/edit post pages
   *
   * @param none
   * @return void
   */
  function post_checkbox() {
   global $post;

   wp_nonce_field( plugin_basename(__FILE__), 'wordpress-https' );

   $checked = false;
   if ($post->ID) {
    $checked = get_post_meta($post->ID, 'force_ssl', true);
   }
   echo '<div class="misc-pub-section misc-pub-section-last" style="border-top: 1px solid #EEE;"><label>Force SSL: <input type="checkbox" value="1" name="force_ssl" id="force_ssl"'.(($checked) ? ' checked="checked"' : '').' /></label></div>';
  }

  /**
   * Save Force SSL option to post or page
   *
   * @param int $post_id
   * @return int $post_id
   */
  function post_save( $post_id ) {
   if (array_key_exists('wordpress-https',$_POST)) {
    if ( !wp_verify_nonce( $_POST['wordpress-https'], plugin_basename(__FILE__) )) {
      return $post_id;
    }

    if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) {
     return $post_id;
    }

    if ( 'page' == $_POST['post_type'] ) {
     if ( !current_user_can( 'edit_page', $post_id ) ) {
      return $post_id;
     }
    } else {
     if ( !current_user_can( 'edit_post', $post_id ) ) {
      return $post_id;
     }
    }

    $forceSSL = (($_POST['force_ssl'] == 1) ? true : false);
    if ($forceSSL) {
     update_post_meta($post_id, 'force_ssl', 1);
    } else {
     delete_post_meta($post_id, 'force_ssl');
    }

    return $forceSSL;
   }
   return $post_id;
  }

  /**
   * Filters HTTPS urls from bloginfo function
   *
   * @param string $result
   * @param string $show
   * @return string
   */
  function bloginfo($result = '', $show = '') {
   if ($show == 'stylesheet_url' || $show == 'template_url' || $show == 'wpurl' || $show == 'home' || $show == 'siteurl' || $show == 'url') {
    $result = $this->replace_https($result);
   }
   return $result;
  }

  /**
   * Add admin panel menu option
   *
   * @param none
   * @return void
   */
  function menu() {
   add_options_page('WordPress HTTPS Settings', 'WordPress HTTPS', 'manage_options', 'wordpress-https', array(&$this, 'settings'));
  }

  /**
   * Settings page in admin panel
   *
   * @param none
   * @return void
   */
  function settings() {
   if ( !current_user_can('manage_options') ) {
    wp_die( __('You do not have sufficient permissions to access this page.') );
   }

   if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
    $errors = array();

    foreach ($this->options_default as $key => $default) {
     if (!array_key_exists($key, $_POST) && $default == 0) {
      $_POST[$key] = 0;
     } else {
      if ( $key == 'wordpress-https_bypass' && @ini_get('allow_url_fopen') != 1 ) {
       $errors[] = 'PHP configuration error: allow_url_fopen must be enabled to allow the conversion of external elements to HTTPS.';
       $_POST[$key] = 0;
      } else if ($key == 'wordpress-https_disable_autohttps' && version_compare(get_bloginfo('version'),'3.0','<')) {
       $_POST[$key] = 0;
      }
     }
     update_option($key, $_POST[$key]);
    }
    if ( @$_POST['ajax'] == 1 ) {
     ob_clean();
     echo "1";
     exit();
    }
   }
?>
 <div class="wrap">
  <div id="icon-options-general" class="icon32"><br /></div>
  <h2>WordPress HTTPS Settings</h2>

<?php
   if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
    if ( sizeof( $errors ) > 0 ) {
     echo "  <ul id=\"message\">\n";
     foreach ( $errors as $error ) {
      echo "   <li class=\"error\"><p>".$error."</p></li>\n";
     }
     echo "  </ul>\n";
    } else {
     echo "   <div class=\"updated below-h2 fade\" id=\"message\"><p>Settings saved.</p></div>\n";
    }
   } else {
    echo "  <div id=\"message-wrap\"><div id=\"message-body\"></div></div>\n";
   }
?>

  <div id="wphttps-sidebar">

   <div class="wphttps-widget" id="wphttps-support">
    <h3 class="wphttps-widget-title">Support</h3>
    <div class="wphttps-widget-content">
     <p>Have you tried everything and your website is still giving you partially encrypted errors?</p>
     <p>If you haven't already, check out the <a href="http://wordpress.org/extend/plugins/wordpress-https/faq/" target="_blank">Frequently Asked Questions</a>.</p>
     <p>Still not fixed? Having other problems? Please <a href="http://wordpress.org/tags/wordpress-https#postform" target="_blank">start a support topic</a> and I'll do my best to assist you.</p>
    </div>
   </div>

   <div class="wphttps-widget" id="wphttps-donate">
    <h3 class="wphttps-widget-title">Donate</h3>
    <div class="wphttps-widget-content">
     <p>If you found this plugin useful, or I've already helped you with your website, please considering buying me a <a href="http://en.wikipedia.org/wiki/Newcastle_Brown_Ale" target="_blank">beer</a> or two.</p>
     <p>Donations help alleviate the time spent developing and supporting this plugin and are greatly appreciated.</p>

     <form name="_xclick" action="https://www.paypal.com/cgi-bin/webscr" method="post">
      <input type="hidden" name="cmd" value="_xclick">
      <input type="hidden" name="business" value="mike@mvied.com">
      <input type="hidden" name="item_name" value="WordPress HTTPS Donation">
      <input type="hidden" name="currency_code" value="USD">
      <label for="amount">Amount: $ <input type="text" name="amount" value="0.00" /></label>
      <input type="submit" class="button-secondary" value="Donate" />
     </form>
    </div>
   </div>

  </div>

  <form name="form" id="wordpress-https" action="options-general.php?page=wordpress-https" method="post">
   <?php settings_fields('wordpress-https'); ?>

   <fieldset>
    <label for="wordpress-https_internalurls"><input name="wordpress-https_internalurls" type="checkbox" id="wordpress-https_internalurls" value="1"<?php echo ((get_option('wordpress-https_internalurls')) ? ' checked="checked"' : ''); ?> /> <strong>Internal HTTPS Elements</strong></label>
    <p>Force internal elements to HTTPS.</p>
    <p class="description">Disabling this will basically disable the plugin.</p>
   </fieldset>

   <fieldset>
    <label for="wordpress-https_externalurls"><input name="wordpress-https_externalurls" type="checkbox" id="wordpress-https_externalurls" value="1"<?php echo ((get_option('wordpress-https_externalurls')) ? ' checked="checked"' : ''); ?> /> <strong>External HTTPS Elements</strong></label>
    <p>Attempt to automatically force external elements to HTTPS.</p>
    <p class="description">Warning: This option checks that the external element can be loaded via HTTPS while the page is loading. Depending on the amount of external elements, this could affect the load times of your pages.</p>
   </fieldset>

   <fieldset>
    <label for="wordpress-https_bypass"><input name="wordpress-https_bypass" type="checkbox" id="wordpress-https_bypass" value="1"<?php echo ((get_option('wordpress-https_bypass')) ? ' checked="checked"' : ''); ?> /> <strong>Bypass External Check</strong></label>
    <p>Disable the option to check if an external element can be loaded over HTTPS.</p>
    <p class="description">Warning: Bypassing the HTTPS check for external elements may cause elements to not load at all. Only enable this option if you know that all external elements can be loaded over HTTPS.</p>
   </fieldset>

<?php if (version_compare(get_bloginfo('version'),'3.0','>=')) { ?>
   <fieldset>
    <label for="wordpress-https_disable_autohttps"><input name="wordpress-https_disable_autohttps" type="checkbox" id="wordpress-https_disable_autohttps" value="1"<?php echo ((get_option('wordpress-https_disable_autohttps')) ? ' checked="checked"' : ''); ?> /> <strong>Disable Automatic HTTPS</strong></label>
    <p>Disable the feature in WordPress 3.0+ that makes all links HTTPS if the page is being viewed via HTTPS.</p>
    <p class="description">When a page is viewed via HTTPS in WordPress 3.0+, all internal page, category and post links are forced to HTTPS. This option will disable that.</p>
   </fieldset>

<?php } ?>

   <fieldset>
    <label for="wordpress-https_exclusive_https"><input name="wordpress-https_exclusive_https" type="checkbox" id="wordpress-https_exclusive_https" value="1"<?php echo ((get_option('wordpress-https_exclusive_https')) ? ' checked="checked"' : ''); ?> /> <strong>Force SSL Exclusively</strong></label>
    <p>Exclusively force SSL on posts and pages with the `Force SSL` option checked. All others are redirected to HTTP.</p>
    <p class="description">WordPress HTTPS adds a 'Force SSL' checkbox to each post and page right above the publish button (<a href="<?php echo $this->plugin_url; ?>screenshot-2.png" target="_blank">screenshot</a>). When selected, the post or page will be forced to HTTPS. With this option enabled, all posts and pages without 'Force SSL' checked will be redirected to HTTP.</p>
   </fieldset>

   <fieldset>
    <label for="wordpress-https_sharedssl"><input name="wordpress-https_sharedssl" type="checkbox" id="wordpress-https_sharedssl" value="1"<?php echo ((get_option('wordpress-https_sharedssl')) ? ' checked="checked"' : ''); ?> /> <strong>Shared SSL</strong></label>
    <p>Enable this option if you are using a shared SSL certificate. Be sure to type your Shared SSL Host below.</p>
    <label><strong>Shared SSL Host</strong> <input name="wordpress-https_sharedssl_host" type="text" id="wordpress-https_sharedssl_host" value="<?php echo get_option('wordpress-https_sharedssl_host'); ?>" /></label>
   </fieldset>

<?php if (get_option('show_on_front') == 'posts') { ?>
   <fieldset>
    <label for="wordpress-https_frontpage"><input name="wordpress-https_frontpage" type="checkbox" id="wordpress-https_frontpage" value="1"<?php echo ((get_option('wordpress-https_frontpage')) ? ' checked="checked"' : ''); ?> /> <strong>HTTPS Front Page</strong></label>
    <p>It appears you are using your latest posts for your home page. If you would like that page to have SSL enforced, enable this option.</p>
   </fieldset>

<?php } ?>

   <p class="button-controls">
    <input type="submit" name="Submit" value="Save Changes" class="button-primary" />
    <img alt="Waiting..." src="<?php echo $this->plugin_url; ?>/css/images/wpspin_light.gif" class="waiting" id="submit-waiting" />
   </p>
  </form>

  <br style="clear: both" />

 </div>
<?php
  }

  function plugin_links($links,$file) {
   if ( strpos($file,basename( __FILE__)) === false ) {
    return $links;
   }

   $links[] = '<a href="'.site_url().'/wp-admin/options-general.php?page=wordpress-https" title="WordPress HTTPS Settings">Settings</a>';
   $links[] = '<a href="http://wordpress.org/extend/plugins/wordpress-https/faq/" title="Frequently Asked Questions">FAQ</a>';
   $links[] = '<a href="http://wordpress.org/tags/wordpress-https#postform" title="Support">Support</a>';
   $links[] = '<a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=6ZL95VTJ388HG" title="Support WordPress HTTPS development with a donation!">Donate</a>';
   return $links;
  }

  /**
   * Start output buffering
   *
   * @param none
   * @return void
   */
  function buffer_start() {
   if ( get_option('wordpress-https_externalurls') == 1 && get_option('wordpress-https_bypass') != 1 ) {
    @ini_set('allow_url_fopen', 1);
   }
   ob_start(array(&$this, 'process'));
  }

  /**
   * End output buffering
   *
   * @param none
   * @return void
   */
  function buffer_end() {
   ob_end_flush();
  }

  /**
   * Replace HTTPS with HTTP
   *
   * @param string
   * @return string
   */
  function replace_https($string) {
   return str_replace('https://', 'http://', $string);
  }

  /**
   * Replace HTTP with HTTPS
   *
   * @param string
   * @return string
   */
  function replace_http($string) {
   return str_replace('http://', 'https://', $string);
  }

 } // End WordPressHTTPS Class
}

if ( class_exists('WordPressHTTPS') ) {
 $wordpress_https = new WordPressHTTPS();
}
?>